const { test, expect } = require( '@playwright/test' );

test.describe( 'Analytics pages', () => {
	test.use( { storageState: 'e2e/storage/adminState.json' } );

	test( 'a user should see 3 sections by default - Performance, Charts, and Leaderboards', async ( {
		page,
	} ) => {
		// Create an array of the sections we're expecting to find.
		const arrExpectedSections = [ 'Charts', 'Leaderboards', 'Performance' ];
		await page.goto(
			'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
		);

		await page.waitForLoadState( 'networkidle' );

		// Grab all of the section headings
		const sections = await page.$$(
			'h2.woocommerce-section-header__title'
		);
		// Create an array with the section headings
		const arrFoundSections = new Array();
		for await ( const section of sections ) {
			arrFoundSections.push( await section.innerText() );
		}
		await expect( arrFoundSections.sort() ).toEqual(
			arrExpectedSections.sort()
		);
	} );

	test.describe( 'moving sections', () => {
		test.use( { storageState: 'e2e/storage/adminState.json' } );

		test( 'should not display move up for the top, or move down for the bottom section', async ( {
			page,
		} ) => {
			await page.goto(
				'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
			);
			// check the top section
			await page.click(
				'//button[@title="Choose which analytics to display and the section name"]'
			);
			await expect( page.locator( 'text=Move up' ) ).not.toBeVisible();
			await expect( page.locator( 'text=Move down' ) ).toBeVisible();

			// check the bottom section
			await page.click(
				'//button[@title="Choose which leaderboards to display and other settings"]'
			);
			await expect( page.locator( 'text=Move down' ) ).not.toBeVisible();
			await expect( page.locator( 'text=Move up' ) ).toBeVisible();
		} );

		test( 'should allow a user to move a section down', async ( {
			page,
		} ) => {
			await page.goto(
				'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
			);
			const firstSection = await page
				.locator( 'h2.woocommerce-section-header__title >> nth=0' )
				.innerText();
			const secondSection = await page
				.locator( 'h2.woocommerce-section-header__title >> nth=1' )
				.innerText();

			await page.click(
				'button.components-button.woocommerce-ellipsis-menu__toggle >> nth=0'
			);
			await page.click( 'text=Move down' );

			// second section becomes first section, first becomes second
			await expect(
				page.locator( 'h2.woocommerce-section-header__title >> nth=0' )
			).toHaveText( secondSection );
			await expect(
				page.locator( 'h2.woocommerce-section-header__title >> nth=1' )
			).toHaveText( firstSection );
		} );

		test( 'should allow a user to move a section up', async ( {
			page,
		} ) => {
			await page.goto(
				'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
			);
			const firstSection = await page
				.locator( 'h2.woocommerce-section-header__title >> nth=0' )
				.innerText();
			const secondSection = await page
				.locator( 'h2.woocommerce-section-header__title >> nth=1' )
				.innerText();

			await page.click(
				'button.components-button.woocommerce-ellipsis-menu__toggle >> nth=1'
			);
			await page.click( 'text=Move up' );

			// second section becomes first section, first becomes second
			await expect(
				page.locator( 'h2.woocommerce-section-header__title >> nth=0' )
			).toHaveText( secondSection );
			await expect(
				page.locator( 'h2.woocommerce-section-header__title >> nth=1' )
			).toHaveText( firstSection );
		} );
	} );

	test( 'should allow a user to remove a section', async ( { page } ) => {
		await page.goto(
			'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
		);
		// clicks the first button to the right of the Performance heading
		await page.click( 'button:right-of(:text("Performance")) >> nth=0' );
		await page.click( 'text=Remove section' );
		// Grab all of the section headings
		const sections = await page.$$(
			'h2.woocommerce-section-header__title'
		);
		await expect( sections.length ).toEqual( 2 );

		// clean up
		await page.click( '//button[@title="Add more sections"]' );
		await page.click( '//button[@title="Add Performance section"]' );
		await page.waitForSelector( 'h2:has-text("Performance")', {
			state: 'visible',
		} );
		await page.waitForLoadState( 'networkidle' );
	} );

	test( 'should allow a user to add a section back in', async ( {
		page,
	} ) => {
		await page.goto(
			'wp-admin/admin.php?page=wc-admin&path=%2Fanalytics%2Foverview'
		);
		// button only shows when not all sections visible, so remove a section
		await page.click( 'button:right-of(:text("Performance")) >> nth=0' );
		await page.click( 'text=Remove section' );

		// add section
		await page.click( '//button[@title="Add more sections"]' );
		await page.click( '//button[@title="Add Performance section"]' );
		await expect(
			page.locator( 'h2.woocommerce-section-header__title >> nth=2' )
		).toContainText( 'Performance' );

		// clean up by moving performance section back to the top
		await page.click(
			'//button[@title="Choose which analytics to display and the section name"]'
		);
		await page.click( 'text=Move up' );
		await page.click(
			'//button[@title="Choose which analytics to display and the section name"]'
		);
		await page.click( 'text=Move up' );
	} );
} );
